# ################################################################# #
#### LOAD LIBRARY AND DEFINE CORE SETTINGS                       ####
# ################################################################# #

### Clear memory and initialise
rm(list = ls())
library(apollo)
apollo_initialise()

### Set core controls
apollo_control = list(
  modelName  ="eMDC2",
  modelDescr ="Extended MDC with complementarity and substitution, with unobserved budget and socio-demographics",
  indivID    ="indivID",
  outputDirectory="output"
)

# ################################################################# #
#### LOAD DATA AND APPLY ANY TRANSFORMATIONS                     ####
# ################################################################# #

### Load data from within the Apollo package
db = apollo_timeUseData

### Create consumption variables for combined activities
# outside good: time spent at home and travelling
db$t_outside = rowSums(db[,c("t_a01", "t_a06", "t_a10", "t_a11", "t_a12")]) 
db$t_leisure = rowSums(db[,c("t_a07", "t_a08", "t_a09")])

### Randomly split dataset into estimation (70%) and validation (30%)
set.seed(1)
db$validation <- runif(nrow(db))>0.7
dbVal <- db[ db$validation,] # validation sample
db    <- db[!db$validation,] # estimation sample

# ################################################################# #
#### DEFINE MODEL PARAMETERS                                     ####
# ################################################################# #

### Parameters starting values c(name1=value1, name2=value2, ...)
apollo_beta = c(aFemale      = 0, 
                # Base utility of inside goods
                bWork        = 0, bSchool      = 0, 
                bShopping    = 0, bPrivate     = 0, 
                bLeisure     = 0, bWork_FT     = 0, 
                bWork_wknd   = 0, bSchool_young= 0, 
                bLeisure_wknd= 0, 
                # Satiation
                gWork        = 1, gSchool      = 1, 
                gShopping    = 1, gPrivate     = 1, 
                gLeisure     = 1, 
                dWorkScho    = 0, 
                # Complementarity and substitution
                dWorkShop    = 0, dWorkPriv    = 0, 
                dWorkLeis    = 0, dSchoShop    = 0, 
                dSchoPriv    = 0, dSchoLeis    = 0, 
                dShopPriv    = 0, dShopLeis    = 0, 
                dPrivLeis    = 0,
                # S.d. of error term
                sigma        = 1)

### Names of fixed parameters
apollo_fixed = c('dWorkShop', 'dWorkPriv', 'dSchoShop', 
                 'dSchoPriv', 'dSchoLeis', 'dWorkLeis')

# ################################################################# #
#### GROUP AND VALIDATE INPUTS                                   ####
# ################################################################# #

apollo_inputs = apollo_validateInputs(database=db)
source("apollo_emdc2.R")

# ################################################################# #
#### DEFINE MODEL AND LIKELIHOOD FUNCTION                        ####
# ################################################################# #

apollo_probabilities=function(apollo_beta, apollo_inputs, 
                              functionality="estimate"){
    
  ### Initialise
  apollo_attach(apollo_beta, apollo_inputs)
  on.exit(apollo_detach(apollo_beta, apollo_inputs))
  P = list()
  
  ### Prepare Inputs
  alts  = c("work", "school", "shopping", "private", "leisure")
  nAlt = length(alts)
  A = setNames(as.list(rep(1, nAlt)), alts)
  X = list(work     =     t_a02/60,
           school   =     t_a03/60,
           shopping =     t_a04/60,
           private  =     t_a05/60,
           leisure  = t_leisure/60)
  V = list(
    work     = bWork     + bWork_FT*occ_full_time + bWork_wknd*weekend,
    school   = bSchool   + bSchool_young*(age<=30), 
    shopping = bShopping, 
    private  = bPrivate, 
    leisure  = bLeisure  + bLeisure_wknd*weekend
  )
  G = list(work     = gWork,    
           school   = gSchool,
           shopping = gShopping,
           private  = gPrivate,
           leisure  = gLeisure)
  D <- c(0,                 0,         0,         0, 0,
         dWorkScho,         0,         0,         0, 0,
         dWorkShop, dSchoShop,         0,         0, 0,
         dWorkPriv, dSchoPriv, dShopPriv,         0, 0,
         dWorkLeis, dSchoLeis, dShopLeis, dPrivLeis, 0)
  D <- matrix(D, nrow=nAlt, ncol=nAlt, byrow=TRUE)
  emdc_settings <- list(X = X, 
                        A = A,
                        V0= aFemale*female, 
                        V = V, 
                        gamma = G, 
                        delta = D, 
                        cost  = A,
                        sigma = sigma)
  P[["model"]] = apollo_emdc2(emdc_settings, functionality)
  
  ### Comment out as necessary
  P = apollo_panelProd(P, apollo_inputs, functionality)
  P = apollo_prepareProb(P, apollo_inputs, functionality)
  return(P)
}

# ################################################################# #
#### MODEL ESTIMATION & OUTPUT                                   ####
# ################################################################# #

model = apollo_estimate(apollo_beta, apollo_fixed, 
                        apollo_probabilities, apollo_inputs)

apollo_modelOutput(model)

apollo_saveOutput(model)

# ################################################################# #
#### PREDICTION                                                  ####
# ################################################################# #

model <- apollo_loadModel(apollo_control$modelName)
apollo_inputs <- apollo_validateInputs(database=dbVal)
pred <- apollo_prediction(model, apollo_probabilities, apollo_inputs)
XObs <- cbind(work     =     dbVal$t_a02/60,
              school   =     dbVal$t_a03/60,
              shopping =     dbVal$t_a04/60,
              private  =     dbVal$t_a05/60,
              leisure  = dbVal$t_leisure/60)
XPre <- pred[,3:7]

round(sqrt(colMeans((XObs - XPre)^2)),2) # RMSE per product: 3.52 0.67 1.66 1.97 3.86
round(sqrt(mean((colSums(XObs) - colSums(XPre))^2)),2) # RMSE for sample: 130.55
