# ################################################################# #
#### LOAD LIBRARY AND DEFINE CORE SETTINGS                       ####
# ################################################################# #

### Clear memory and initialise
rm(list = ls())
library(apollo)
apollo_initialise()

### Set core controls
apollo_control = list(
  modelName  ="eMDC1",
  modelDescr ="Extended MDC with complementarity and substitution, with observed budget and socio-demographics",
  indivID    ="indivID",
  outputDirectory="output"
)

# ################################################################# #
#### LOAD DATA AND APPLY ANY TRANSFORMATIONS                     ####
# ################################################################# #

### Load data from within the Apollo package
db = apollo_timeUseData

### Create consumption variables for combined activities
# outside good: time spent at home and travelling
db$t_outside = rowSums(db[,c("t_a01", "t_a06", "t_a10", "t_a11", "t_a12")]) 
db$t_leisure = rowSums(db[,c("t_a07", "t_a08", "t_a09")])

### Randomly split dataset into estimation (70%) and validation (30%)
set.seed(1)
db$validation <- runif(nrow(db))>0.7
dbVal <- db[ db$validation,] # validation sample
db    <- db[!db$validation,] # estimation sample

# ################################################################# #
#### DEFINE MODEL PARAMETERS                                     ####
# ################################################################# #

### Parameters starting values c(name1=value1, name2=value2, ...)
apollo_beta = c(aFemale=-0.005, sigma = 2, 
                # Satiation
                gWork    = 10.425, 
                gSchool  = 9.955, 
                gShopping= 2.065, 
                gPrivate = 4.419, 
                gLeisure = 6.927, 
                # Base utility
                bWork     =-3.372, bSchool   =-5.250, 
                bShopping =-3.663, bPrivate  =-3.973, 
                bLeisure  =-3.300, bWork_FT  = 0.708, 
                bWork_wknd=-1.600, bSchool_young= 0.948, 
                bLeisure_wknd= 0.155, 
                # Compl/subst
                dWorkScho=-0.008, dWorkShop= 0.000, 
                dWorkPriv= 0.000, dWorkLeis= 0.000, 
                dSchoShop= 0.000, dSchoPriv= 0.000, 
                dSchoLeis= 0.000, dShopPriv= 0.010, 
                dShopLeis= 0.012, dPrivLeis= 0.012)

### Names of fixed parameters
apollo_fixed = c('dWorkShop', 'dWorkPriv', 'dSchoShop', 
                 'dSchoPriv', 'dSchoLeis', 'dWorkLeis')

# ################################################################# #
#### GROUP AND VALIDATE INPUTS                                   ####
# ################################################################# #

apollo_inputs = apollo_validateInputs(database=db)
source("apollo_emdc1.R")

# ################################################################# #
#### DEFINE MODEL AND LIKELIHOOD FUNCTION                        ####
# ################################################################# #

apollo_probabilities=function(apollo_beta, apollo_inputs, 
                              functionality="estimate"){
    
  ### Initialise
  apollo_attach(apollo_beta, apollo_inputs)
  on.exit(apollo_detach(apollo_beta, apollo_inputs))
  P = list()
  
  ### Prepare Inputs
  alts  = c("work", "school", "shopping", "private", "leisure")
  nAlt = length(alts)
  A = setNames(as.list(rep(1, nAlt)), alts)
  X = list(work     =     t_a02/60,
           school   =     t_a03/60,
           shopping =     t_a04/60,
           private  =     t_a05/60,
           leisure  = t_leisure/60)
  V = list(
    work     = bWork     + bWork_FT*occ_full_time + bWork_wknd*weekend,
    school   = bSchool   + bSchool_young*(age<=30), 
    shopping = bShopping, 
    private  = bPrivate, 
    leisure  = bLeisure  + bLeisure_wknd*weekend
  )
  G = list(work     = gWork,    
           school   = gSchool,
           shopping = gShopping,
           private  = gPrivate,
           leisure  = gLeisure)
  D <- c(0,                 0,         0,         0, 0,
         dWorkScho,         0,         0,         0, 0,
         dWorkShop, dSchoShop,         0,         0, 0,
         dWorkPriv, dSchoPriv, dShopPriv,         0, 0,
         dWorkLeis, dSchoLeis, dShopLeis, dPrivLeis, 0)
  D <- matrix(D, nrow=nAlt, ncol=nAlt, byrow=TRUE)
  emdc_settings <- list(X = X, 
                        A = A,
                        V0= aFemale*female, 
                        V = V, 
                        sigma = sigma, 
                        B = 24, 
                        gamma = G, 
                        delta = D, 
                        cost  = A)
  P[["model"]] = apollo_emdc1(emdc_settings, functionality)
  
  ### Comment out as necessary
  P = apollo_panelProd(P, apollo_inputs, functionality)
  P = apollo_prepareProb(P, apollo_inputs, functionality)
  return(P)
}

# ################################################################# #
#### MODEL ESTIMATION & OUTPUT                                   ####
# ################################################################# #

model = apollo_estimate(apollo_beta, apollo_fixed, 
                        apollo_probabilities, apollo_inputs)

apollo_modelOutput(model)

apollo_saveOutput(model)


# ################################################################# #
#### PREDICTION                                                  ####
# ################################################################# #

model <- apollo_loadModel(apollo_control$modelName)
apollo_inputs <- apollo_validateInputs(database=dbVal)
pred <- apollo_prediction(model, apollo_probabilities, apollo_inputs)
XObs <- cbind(work     =     dbVal$t_a02/60,
              school   =     dbVal$t_a03/60,
              shopping =     dbVal$t_a04/60,
              private  =     dbVal$t_a05/60,
              leisure  = dbVal$t_leisure/60)
XPre <- pred[,4:8]

round(sqrt(colMeans((XObs - XPre)^2)),2) # RMSE per product: 3.62 0.68 1.67 1.98 3.90
round(sqrt(mean((colSums(XObs) - colSums(XPre))^2)),2) # RMSE for sample: 52.15
